
/******************************************************************************
ļ  :
  :
  :
޸  :

******************************************************************************/

#include "Drv_CommuSPI2.h"

osMutexId CommuSPI2Mutex = NULL;

osMutexId GetCommuMutex()
{
  return CommuSPI2Mutex;
}

/**
  * @brief  Sends a byte through the SPI interface and return the byte received
  *         from the SPI bus.
  * @param  byte: byte to send.
  * @retval The value of the received byte.
  */
u8 CommuSPI2_SendByte(u8 byte)
{
  /*!< Loop while DR register in not emplty */
  while (SPI_I2S_GetFlagStatus(CommuSPI2, SPI_I2S_FLAG_TXE) == RESET);

  /*!< Send byte through the SPI1 peripheral */
  SPI_I2S_SendData(CommuSPI2, byte);

  /*!< Wait to receive a byte */
  while (SPI_I2S_GetFlagStatus(CommuSPI2, SPI_I2S_FLAG_RXNE) == RESET);

  /*!< Return the byte read from the SPI bus */
  return SPI_I2S_ReceiveData(CommuSPI2);
}

/**
  * @brief  Reads a byte from the SPI Flash.
  * @note   This function must be used only if the Start_Read_Sequence function
  *         has been previously called.
  * @param  None
  * @retval Byte Read from the SPI Flash.
  */
u8 CommuSPI2_ReadByte(u8 DUMMY_BYTE)
{
  return (CommuSPI2_SendByte(DUMMY_BYTE));
}

/**
  * @brief  Sends a Half Word through the SPI interface and return the Half Word
  *         received from the SPI bus.
  * @param  HalfWord: Half Word to send.
  * @retval The value of the received Half Word.
  */
u16 CommuSPI2_SendHalfWord(u16 HalfWord)
{
  /*!< Loop while DR register in not emplty */
  while (SPI_I2S_GetFlagStatus(CommuSPI2, SPI_I2S_FLAG_TXE) == RESET);

  /*!< Send Half Word through the sFLASH peripheral */
  SPI_I2S_SendData(CommuSPI2, HalfWord);

  /*!< Wait to receive a Half Word */
  while (SPI_I2S_GetFlagStatus(CommuSPI2, SPI_I2S_FLAG_RXNE) == RESET);

  /*!< Return the Half Word read from the SPI bus */
  return SPI_I2S_ReceiveData(CommuSPI2);
}
#if 0
/**
  * @brief  DeInitializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
void CommuSPI2_LowLevel_DeInit(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /*!< Disable the CommuSPI2  ************************************************/
  SPI_Cmd(CommuSPI2, DISABLE);

  /*!< DeInitializes the CommuSPI2 *******************************************/
  SPI_I2S_DeInit(CommuSPI2);

  /*!< CommuSPI2 Periph clock disable ****************************************/
  CommuSPI2_CLK_INIT(CommuSPI2_CLK, DISABLE);

  /*!< Configure all pins used by the SPI as input floating *******************/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;

  GPIO_InitStructure.GPIO_Pin = CommuSPI2_SCK_PIN;
  GPIO_Init(CommuSPI2_SCK_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = CommuSPI2_MISO_PIN;
  GPIO_Init(CommuSPI2_MISO_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = CommuSPI2_MOSI_PIN;
  GPIO_Init(CommuSPI2_MOSI_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = sFLASH_CS_PIN;
  GPIO_Init(sFLASH_CS_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @brief  DeInitializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
void CommuSPI2_DeInit(void)
{
  CommuSPI2_LowLevel_DeInit();
}
#endif

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/


/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Initializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
void CommuSPI2_LowLevel_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  SPI_InitTypeDef  SPI_InitStructure;

//  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
//  GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE);      /*ʹSWD JTAG*/

  /*!< Enable the SPI clock */
  CommuSPI2_CLK_INIT(CommuSPI2_CLK, ENABLE);

  /*!< Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(CommuSPI2_SCK_GPIO_CLK | CommuSPI2_MISO_GPIO_CLK |
                         CommuSPI2_MOSI_GPIO_CLK | sFLASH_CS_GPIO_CLK, ENABLE);

  /*!< SPI pins configuration *************************************************/

  /*!< Connect SPI pins to AF5 */
  GPIO_PinAFConfig(CommuSPI2_SCK_GPIO_PORT, CommuSPI2_SCK_SOURCE, CommuSPI2_SCK_AF);
  GPIO_PinAFConfig(CommuSPI2_MISO_GPIO_PORT, CommuSPI2_MISO_SOURCE, CommuSPI2_MISO_AF);
  GPIO_PinAFConfig(CommuSPI2_MOSI_GPIO_PORT, CommuSPI2_MOSI_SOURCE, CommuSPI2_MOSI_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;

  /*!< SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = CommuSPI2_SCK_PIN;
  GPIO_Init(CommuSPI2_SCK_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  CommuSPI2_MOSI_PIN;
  GPIO_Init(CommuSPI2_MOSI_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MISO pin configuration */
  GPIO_InitStructure.GPIO_Pin =  CommuSPI2_MISO_PIN;
  GPIO_Init(CommuSPI2_MISO_GPIO_PORT, &GPIO_InitStructure);

  /*!< Configure sFLASH Card CS pin in output pushpull mode ********************/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = sFLASH_CS_PIN;
  GPIO_Init(sFLASH_CS_GPIO_PORT, &GPIO_InitStructure);
  GPIO_SetBits(sFLASH_CS_GPIO_PORT, sFLASH_CS_PIN);
  
   /*!< Configure sFLASH Card rest pin in output pushpull mode ********************/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_SetBits(GPIOB,  GPIO_Pin_11);
  /*!< Configure W5500 Card CS pin in output pushpull mode ********************/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = W5500_CS_PIN;
  GPIO_Init(W5500_CS_GPIO_PORT, &GPIO_InitStructure);
  GPIO_SetBits(W5500_CS_GPIO_PORT, W5500_CS_PIN);

  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;//SPI_CPOL_Low;//
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;//SPI_CPHA_1Edge;//
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(CommuSPI2, &SPI_InitStructure);

  /*!< Enable the sFLASH_SPI  */
  SPI_Cmd(CommuSPI2, ENABLE);

  CommuSPI2Mutex = NULL;
  osMutexDef(CommuSPI2Mutex);
  CommuSPI2Mutex = osMutexCreate(osMutex(CommuSPI2Mutex));
}

