
#include "Drv_GprsCommon.h"

extern StructGprsDrv Sim800cDrv;

StructGprsDrv *pGprsDrvHandle[] =
{
  &Sim800cDrv,
  NULL
};

StructGprsSta Gprs;

/******************************************************************************
  : Module_USARTx_IRQHandler
  : ڽж
  : 
  : 
******************************************************************************/
void Gprs_USARTx_IRQHandler(void)
{
  u16 CR1,SR;
  u8 Data;
  osEvent SendDataMsg;

  SR = Gprs_USARTx->SR;
  CR1 = Gprs_USARTx->CR1;
  if((CR1 & USART_CR1_RXNEIE) && (SR & USART_SR_RXNE))
  {
    Data = (u8)Gprs_USARTx->DR;
    osMessagePut(Gprs.Uart.RecvQ, Data, 0);
  }

  if((CR1 & USART_CR1_TXEIE) && (SR & USART_SR_TXE))
  {
    SendDataMsg = osMessageGet(Gprs.Uart.SendQ, 0);
    if(osEventMessage == SendDataMsg.status)
      Gprs_USARTx->DR = SendDataMsg.value.v;
    else
      Gprs_USARTx->CR1 &= ~USART_CR1_TXEIE;
  }
}

/******************************************************************************
  : GprsModSendByte
  : ֽ
  : Data - յֽ
  : TRUE - ɹ
        FALSE - ʧ
******************************************************************************/
u8 GprsModSendByte(u8 Data)
{
  osStatus EvenStatus;

  EvenStatus = osMessagePut(Gprs.Uart.SendQ, Data, portMAX_DELAY);
  if(osOK == EvenStatus)
  {
    Gprs_USARTx->CR1 |= USART_CR1_TXEIE;
    return TRUE;
  }

  return FALSE;
}

int GprsModPrintf(char *Fmt, ...)
{
  char Buf[256];
  char *pData;
  int Count;
  va_list Args;

  va_start(Args, Fmt);
  Count = vsnprintf(Buf, sizeof(Buf), Fmt, Args);
  pData = Buf;
  while(Count)
  {
    GprsModSendByte(*(u8*)pData);
    pData++;
    Count--;
  }
  va_end(Args);

  return (int)(pData-Buf);
}

int GprsModSendCmd(char *pRespEx, char *Fmt, ...)
{
  char Buf[256];
  char *pData;
  int Count;
  va_list Args;

  Gprs.Cmd.Sta = 0;
  Gprs.Cmd.Ack = 0;
  memset(Gprs.Cmd.RespExBuf, 0, sizeof(Gprs.Cmd.RespExBuf));
  Gprs.Cmd.pRespExFlag = pRespEx;
  Gprs.Cmd.pResp_1 = "OK";
  Gprs.Cmd.pResp_2 = "ERROR";

  va_start(Args, Fmt);
  Count = vsnprintf(Buf, sizeof(Buf), Fmt, Args);

  pData = Buf;
  while(Count)
  {
    GprsModSendByte(*(u8*)pData);
    pData++;
    Count--;
  }
  va_end(Args);

  Gprs.Cmd.Sta = 1;

  return (int)(pData-Buf);
}

u8 GprsModWaitAck(u32 Ms)
{
  u32 Cnt = Ms/13 + 1;

  do
  {
    osDelay(13);
    if(2 == Gprs.Cmd.Sta)
      return Gprs.Cmd.Ack;
    Cnt--;
  }while(Cnt);

  return 0xFF;//ʱ
}

typedef enum
{
  eGprsRespSta_0D = 0,
  eGprsRespSta_0A,
}EnumGprsResp;

/******************************************************************
*    ƣ InternetFrameRecv
*       ȡڽն(FIFO)һ֡
            һ֡EEͷFF FC FF FF βһ֡
*ڲ 
*ڲ 
********************************************************************/
u8 GprsModGetFrame(char *pFrame, u16 MaxLen)
{
  osEvent RecvDataMsg;
  u8 DataTemp;
  u8 FrameLen = 0;
  EnumGprsResp RecvSta = eGprsRespSta_0D;

  RecvDataMsg = osMessageGet(Gprs.Uart.RecvQ, 23);
  while((osEventMessage == RecvDataMsg.status) && (FrameLen < (MaxLen-1)))
  {
    DataTemp = RecvDataMsg.value.v;
    pFrame[FrameLen] = DataTemp;
    FrameLen++;
    pFrame[FrameLen] = 0;
    switch(RecvSta)
    {
      case eGprsRespSta_0D:
        if(0x0D == DataTemp)//յ0x0D
          RecvSta = eGprsRespSta_0A;
        else
          RecvSta = eGprsRespSta_0D;
        break;
      case eGprsRespSta_0A:
        if(0x0A == DataTemp)
        {//յ0x0A,
          pFrame[FrameLen] = 0;
          if(FrameLen > 2)//ݷ
            return FrameLen;
          else
          {//ݣ㣬
            RecvSta = eGprsRespSta_0D;
            FrameLen = 0;
          }
        }
        else if(0x0D == DataTemp)
          RecvSta = eGprsRespSta_0A;
        else
          RecvSta = eGprsRespSta_0D;
        break;

      default:
        if(0x0D == DataTemp)//յ0x0D
          RecvSta = eGprsRespSta_0A;
        else
          RecvSta = eGprsRespSta_0D;
        FrameLen = 0;
        pFrame[FrameLen] = DataTemp;
        FrameLen++;
        pFrame[FrameLen] = 0;
        break;
    }
    RecvDataMsg = osMessageGet(Gprs.Uart.RecvQ, 5);
  }

  pFrame[FrameLen]= 0;
  return FrameLen;
}

typedef enum
{
  eGprsProc_At = 0,//0
  eGprsProc_Gmm,//2
  eGprsProc_Error,//16
}EnumGprsProc;

void GprsProcAt(void)
{
  u8 Ack = 0;

  if(0 == Gprs.Proc.StatusTick)
    Gprs.Proc.StatusTick = GetSystemTick();
  if((GetSystemTick() - Gprs.Proc.StatusTick) > 60000)//60s
  {
    Gprs.Proc.StatusTick = 0;
    Gprs.Proc.ErrCnt = 0;
    Gprs.Proc.Status = eGprsProc_Error;
    return;
  }

  GprsModSendCmd(NULL, "AT\r\n");
  Ack = GprsModWaitAck(3000);
  if(1 == Ack)
  {
    Gprs.Err.Bit.NoMod = 0;
    Gprs.Proc.StatusTick = 0;
    Gprs.Proc.ErrCnt = 0;
    Gprs.Proc.Status = eGprsProc_Gmm;
  }
  else
    Gprs.Err.Bit.NoMod = 1;
}

void GprsProcGmm(void)
{
  u8 Ack = 0;
  char *pStr;
  u8 i;

  GprsModSendCmd(NULL, "AT+GMM\r\n");
  Ack = GprsModWaitAck(300);
  if(1 == Ack)
  {
    Gprs.pGprsDrv = NULL;
    i = 0;
    while(NULL != pGprsDrvHandle[i])
    {
      pStr = strstr(Gprs.Cmd.RespExBuf,pGprsDrvHandle[i]->pModName);
      if(NULL != pStr)
        Gprs.pGprsDrv = pGprsDrvHandle[i];
      i++;
    }
    if(NULL != Gprs.pGprsDrv)
      Gprs.Err.Bit.NoDrv = 0;
    else
      Gprs.Err.Bit.NoDrv = 1;
    Gprs.Err.Bit.NoMod = 0;
    Gprs.Proc.ErrCnt = 0;
    Gprs.Proc.Status = eGprsProc_At;
  }
  else
    Gprs.Proc.ErrCnt++;

  if(Gprs.Proc.ErrCnt > 3)
  {
    Gprs.Proc.ErrCnt = 0;
    Gprs.Proc.Status = eGprsProc_Error;
  }
}

void GprsProcError(void)
{
  GPIO_ResetBits(GprsPowerCtrl_PORT, GprsPowerCtrl_PIN);
  osDelay(10000);
  GPIO_SetBits(GprsPowerCtrl_PORT, GprsPowerCtrl_PIN);
  osDelay(2000);

  Gprs.Proc.StatusTick = 0;
  Gprs.Proc.ErrCnt = 0;
  Gprs.Proc.Status = eGprsProc_At;
}

pGprsProcHandle GprsProcHandle[] =
{
  GprsProcAt,
  GprsProcGmm,
  GprsProcError
};

void GprsSendHandle(void)
{
  if(Gprs.Proc.Status < (sizeof(GprsProcHandle)/sizeof(pGprsProcHandle)))
    GprsProcHandle[Gprs.Proc.Status]();
  else
    Gprs.Proc.Status = 0;

}

void Thd_GprsSend(void const *argument)
{
  Gprs.Proc.Status = 0;

  while(1)
  {
    if(NULL == Gprs.pGprsDrv)
      GprsSendHandle();
    else
      Gprs.pGprsDrv->SendHandle();
    osDelay(37);
  }
}

void GprsRecvHandle(void)
{
  char Frame[128];
  char *pStr;

  if(0 != GprsModGetFrame(Frame, sizeof(Frame)))
  {
    if(1 == Gprs.Cmd.Sta)
    {
      if(NULL != Gprs.Cmd.pRespExFlag)
      {
        pStr = strstr(Frame, Gprs.Cmd.pRespExFlag);
        if(pStr)
        {
          memcpy(Gprs.Cmd.RespExBuf, pStr, sizeof(Gprs.Cmd.RespExBuf));
          Gprs.Cmd.RespExBuf[sizeof(Gprs.Cmd.RespExBuf)-1] = 0;
          pStr = strstr(Gprs.Cmd.pRespExFlag, "+CME ERROR");
          if(!pStr)
            return;
        }
      }

      if(NULL != Gprs.Cmd.pResp_1)
      {
        pStr = strstr(Frame, Gprs.Cmd.pResp_1);
        if(pStr)
        {
          Gprs.Cmd.Ack = 1;
          Gprs.Cmd.Sta = 2;
          return;
        }
      }

      if(NULL != Gprs.Cmd.pResp_2)
      {
        pStr = strstr(Frame, Gprs.Cmd.pResp_2);
        if(pStr)
        {
          Gprs.Cmd.Ack = 2;
          Gprs.Cmd.Sta = 2;
          return;
        }
      }
      if(NULL == Gprs.Cmd.pRespExFlag)
      {
        memcpy(Gprs.Cmd.RespExBuf, Frame, sizeof(Gprs.Cmd.RespExBuf));
        Gprs.Cmd.RespExBuf[sizeof(Gprs.Cmd.RespExBuf)-1] = 0;
      }
    }
  }
}

/******************************************************************************
  : Thread_Lcd
  : ˻߳
  : argument - ̴߳
  : 
******************************************************************************/
void Thd_GprsRecv(void const *argument)
{
  while(1)
  {
    osDelay(43);
    if(NULL == Gprs.pGprsDrv)
      GprsRecvHandle();
    else
      Gprs.pGprsDrv->RecvHandle();
  }
}

/******************************************************************************
  : InternetInit
  : ͨŽӿڳʼ
  : 
  : TRUE - ʼ
        FALSE - ʼ
******************************************************************************/
u8 GprsInit(osMessageQId AppRecvQ)
{
  USART_InitTypeDef USART_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;
  GPIO_InitTypeDef GPIO_InitStructure;

  Gprs.AppRecvQ = AppRecvQ;
  CommuUartIrqSet(Gprs_USARTx_IRQHandler);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO,ENABLE);
  GPIO_PinRemapConfig(GPIO_FullRemap_USART3, ENABLE);

  /* Enable GPIO clock */
  /* Configure USART Tx and Rx as alternate function push-pull */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Pin = Gprs_USARTx_TX_PIN;
  RCC_APB2PeriphClockCmd(Gprs_USARTx_TX_GPIO_CLK, ENABLE);
  GPIO_Init(Gprs_USARTx_TX_GPIO_PORT, &GPIO_InitStructure);


  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Pin = Gprs_USARTx_RX_PIN;
  RCC_APB2PeriphClockCmd(Gprs_USARTx_RX_GPIO_CLK, ENABLE);
  GPIO_Init(Gprs_USARTx_RX_GPIO_PORT, &GPIO_InitStructure);

  /* Enable USART clock */
  Gprs_USARTx_CLK_INIT(Gprs_USARTx_CLK, ENABLE);

  /* Enable the USART OverSampling by 8 */
  USART_OverSampling8Cmd(Gprs_USARTx, ENABLE);

  /* USARTx configuration ----------------------------------------------------*/
  USART_InitStructure.USART_BaudRate = Module_USARTx_BaudRate;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  USART_Init(Gprs_USARTx, &USART_InitStructure);

  /* Enable the USARTx Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = Gprs_USARTx_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY + 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Create Storage Message Queue */
  Gprs.Uart.RecvQ = NULL;
  osMessageQDef(RecvQ, 2048, u8);
  Gprs.Uart.RecvQ = osMessageCreate(osMessageQ(RecvQ), NULL);

  Gprs.Uart.SendQ = NULL;
  osMessageQDef(SendQ, 1536, u8);
  Gprs.Uart.SendQ = osMessageCreate(osMessageQ(SendQ), NULL);

//  if(NULL == ModuleRecvMsg)
//    return FALSE;

  /* Enable USART */
  USART_Cmd(Gprs_USARTx, ENABLE);

  /* Enable the Tx buffer empty interrupt */
  USART_ITConfig(Gprs_USARTx, USART_IT_RXNE, ENABLE);

  osThreadDef(Thd_GprsRecv, Thd_GprsRecv, osPriorityAboveNormal, 0, 1*configMINIMAL_STACK_SIZE);
  osThreadCreate(osThread(Thd_GprsRecv), NULL);

  osThreadDef(Thd_GprsSend, Thd_GprsSend, osPriorityHigh, 0, 2*configMINIMAL_STACK_SIZE);
  osThreadCreate(osThread(Thd_GprsSend), NULL);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_InitStructure.GPIO_Pin = GprsPowerCtrl_PIN;
  RCC_APB2PeriphClockCmd(GprsPowerCtrl_RCC, ENABLE);
  GPIO_Init(GprsPowerCtrl_PORT, &GPIO_InitStructure);
  GPIO_SetBits(GprsPowerCtrl_PORT, GprsPowerCtrl_PIN);

  osMutexDef(Mutex);
  Gprs.Send.Mutex = osMutexCreate(osMutex(Mutex));

  return TRUE;
}

u8 GprsNetAvail(void)
{
  if(NULL == Gprs.pGprsDrv)
    return FALSE;

  return Gprs.pGprsDrv->NetAvail();
}

int GprsTcpSend(u8 *pData, u16 Len)
{
  int Ret = 0;
  u32 SendTick;

  if(FALSE == GprsNetAvail())
    return -1;

  if(osOK != osMutexWait(Gprs.Send.Mutex, 5000))//5
    return -2;

  if(FALSE == GprsNetAvail())
  {
    osMutexRelease(Gprs.Send.Mutex);
    return -3;
  }

  if(Len > sizeof(Gprs.Send.Buf))
    Len = sizeof(Gprs.Send.Buf);
  memcpy(Gprs.Send.Buf, pData, Len);
  Gprs.Send.BufLen = Len;

  Gprs.Send.Ctrl = eSendCtrl_SendReq;
  SendTick = GetSystemTick();
  while((GetSystemTick() - SendTick) < 10000)
  {
    osDelay(100);
    if(FALSE == GprsNetAvail())
      break;
    if((eSendCtrl_SendDone == Gprs.Send.Ctrl) || (eSendCtrl_SendErr == Gprs.Send.Ctrl))
      break;
  }

  if(eSendCtrl_SendDone == Gprs.Send.Ctrl)
    Ret = 0;
  else
    Ret = -4;

  Gprs.Send.Ctrl = eSendCtrl_None;
  osMutexRelease(Gprs.Send.Mutex);
  return Ret;
}

void GprsTcpClose(void)
{
  Gprs.SelCtrl = eSelfCtrl_CloseSckt;
}

