
#ifndef __DRV_GPRSCOMMON_H__
#define __DRV_GPRSCOMMON_H__

#include "Global.h"
#include "Drv_CommuUart.h"

#define Module_USARTx_BaudRate                (115200)

#define Gprs_USARTx                           CommuUart_USARTx
#define Gprs_USARTx_CLK                       CommuUart_USARTx_CLK
#define Gprs_USARTx_CLK_INIT                  CommuUart_USARTx_CLK_INIT
#define Gprs_USARTx_IRQn                      CommuUart_USARTx_IRQn
//#define Gprs_USARTx_IRQHandler                USART3_IRQHandler

#define Gprs_USARTx_TX_PIN                    CommuUart_USARTx_TX_PIN
#define Gprs_USARTx_TX_GPIO_PORT              CommuUart_USARTx_TX_GPIO_PORT
#define Gprs_USARTx_TX_GPIO_CLK               CommuUart_USARTx_TX_GPIO_CLK

#define Gprs_USARTx_RX_PIN                    CommuUart_USARTx_RX_PIN
#define Gprs_USARTx_RX_GPIO_PORT              CommuUart_USARTx_RX_GPIO_PORT
#define Gprs_USARTx_RX_GPIO_CLK               CommuUart_USARTx_RX_GPIO_CLK

#define GprsPowerCtrl_PIN            GPIO_Pin_1
#define GprsPowerCtrl_PORT           GPIOE
#define GprsPowerCtrl_RCC            RCC_APB2Periph_GPIOE

typedef enum
{
  eSelfCtrl_None = 0,
  eSelfCtrl_CloseSckt,
  eSelfCtrl_ShutPpp,
  eSelfCtrl_Error,
}EnumGprsSelfCtrl;

typedef union
{
  u32 Mux;
  struct
  {
    u32 NoMod : 1;
    u32 NoDrv : 1;
    u32 NoSim : 1;
    u32 NoReg : 1;
    u32 NoAtt : 1;
    u32 NoCop : 1;
    u32 NoCon : 1;
  }Bit;
}StructGprsError;

typedef struct
{
  osMessageQId RecvQ;
  osMessageQId SendQ;
}StructUartCtrl;

#define MaxSizeRespBuf (64)
#define MaxSizeSendBuf (1280)


typedef struct
{
  u8 Sta;//0δʹ  1ѷ 2Ӧ
  u8 Ack;//0δӦ  1ӦpResp_1   1ӦpResp_2
  char RespExBuf[MaxSizeRespBuf];//Ӧ
  char *pRespExFlag;//Ӧݱ־
  char *pResp_1;
  char *pResp_2;
}StructGprsCmd;

typedef enum
{
  eSendCtrl_None = 0,
  eSendCtrl_SendReq,
  eSendCtrl_Sending,
  eSendCtrl_SendDone,
  eSendCtrl_SendErr,
}EnumGprsSendCtrl;

typedef struct
{
  EnumGprsSendCtrl Ctrl;
  u8 ScktNum;
  u8 Buf[MaxSizeSendBuf];
  u16 BufLen;
  osMutexId Mutex;
}StructSend;

typedef struct
{
  u8 ErrCnt;
  u8 Status;
  u32 StatusTick;
//  u32 SelCtrlTick;
}StructGprsProc;

typedef void(*pGprsSendHandle)(void);
typedef void(*pGprsRecvHandle)(void);
typedef u8(*pGprsNetAvail)(void);

typedef struct
{
  char *pModName;
  pGprsSendHandle SendHandle;
  pGprsRecvHandle RecvHandle;
  pGprsNetAvail   NetAvail;
}StructGprsDrv;

typedef struct
{
  u8 Csq;
  u8 MainScktNum;
  EnumGprsSelfCtrl SelCtrl;
  u8 Ccid[20];
  StructGprsError Err;
  StructUartCtrl Uart;
  StructGprsCmd Cmd;
  StructGprsProc Proc;
  StructSend Send;
  osMessageQId AppRecvQ;
  StructGprsDrv *pGprsDrv;
}StructGprsSta;

typedef void(*pGprsProcHandle)(void);

extern StructGprsSta Gprs;
extern u8 GprsModSendByte(u8 Data);
extern int GprsModPrintf(char *Fmt, ...);
extern int GprsModSendCmd(char *pRespEx, char *Fmt, ...);
extern u8 GprsModWaitAck(u32 Ms);



extern u8 GprsInit(osMessageQId AppRecvQ);
extern u8 GprsNetAvail(void);
extern int GprsTcpSend(u8 *pData, u16 Len);
extern void GprsTcpClose(void);

#endif

