#ifndef __DRV_LCD_H__
#define __DRV_LCD_H__

#include "Global.h"

#define LCD_USARTx_BaudRate                  (9600)

#define LCD_USARTx                           USART6
#define LCD_USARTx_CLK                       RCC_APB2Periph_USART6
#define LCD_USARTx_CLK_INIT                  RCC_APB2PeriphClockCmd
#define LCD_USARTx_IRQn                      USART6_IRQn
#define LCD_USARTx_IRQHandler                USART6_IRQHandler

#define LCD_USARTx_TX_PIN                    GPIO_Pin_6
#define LCD_USARTx_TX_GPIO_PORT              GPIOC
#define LCD_USARTx_TX_GPIO_CLK               RCC_AHB1Periph_GPIOC
#define LCD_USARTx_TX_SOURCE                 GPIO_PinSource6
#define LCD_USARTx_TX_AF                     GPIO_AF_USART6

#define LCD_USARTx_RX_PIN                    GPIO_Pin_7
#define LCD_USARTx_RX_GPIO_PORT              GPIOC
#define LCD_USARTx_RX_GPIO_CLK               RCC_AHB1Periph_GPIOC
#define LCD_USARTx_RX_SOURCE                 GPIO_PinSource7
#define LCD_USARTx_RX_AF                     GPIO_AF_USART6

#define LcdFrameHeadH     (0xAA)
#define LcdFrameHeadL     (0xCC)

typedef enum
{
  eLcdCmd_WrReg     = 0x80,
  eLcdCmd_RdReg     = 0x81,
  eLcdCmd_WrVar     = 0x82,
  eLcdCmd_RdVar     = 0x83,
  eLcdCmd_WrCurve   = 0x84,
}EnumLcdCmd;

typedef enum
{
  eLcdRegAddr_Backlight = 0x01,
  eLcdRegAddr_Buzzer    = 0x02,
  eLcdRegAddr_Page      = 0x03,
  eLcdRegAddr_Rtc       = 0x20,
  eLcdRegAddr_TouchCor  = 0xEA,
  eLcdRegAddr_Reset     = 0xEE,
}EnumRegAddr;


#pragma pack(1)
typedef struct
{
  u8 HeadH;  //֡ͷֽ
  u8 HeadL;  //֡ͷֽ
  u8 FrameLen;    //֡
  u8 FrameCmd;    //֡
  union  //ƽṹ
  {
    struct
    {
      u8 Addr;  //ַ
      u8 Data[64];//
    }WrReg;//дĴ

    struct
    {
      u8 Addr;  //ַ
      u8 Len;   //ݳ
      u8 Data[64];//
    }RdReg;//Ĵ

    struct
    {
      u8 AddrH; //ַ
      u8 AddrL; //ַ
      u8 Data[64];//
    }WrVar;//

    struct
    {
      u8 AddrH; //ַ
      u8 AddrL; //ַ
      u8 Len;   //ݳȣλΪword
      u8 Data[64];
    }RdVar;//

    u8 FrameData[64];
  }Ctrl;
}StructLcdDwinFrame;
#pragma pack()

extern u8 LcdInit(void);
/******************************************************************
*    ƣ LcdFrameRecv
*       ȡڽն(FIFO)һ֡
*ڲ 
*ڲ 
********************************************************************/
extern u8 LcdFrameRecv(StructLcdDwinFrame *pFrame);

/******************************************************************************
  : LcdWriteReg
  : LcdдĴ
  : Addr - дַ
        pData - ͵ָ
        Len - ͵ݳ
  : 
******************************************************************************/
extern void LcdWriteReg(u8 Addr, u8 *pData, u8 Len);

/******************************************************************************
  : LcdReadReg
  : LcdĴ,Ȳܳ64,ͷļ֡ṹ
  : Addr - ַ
        Len - ݳ
  : 
******************************************************************************/
extern void LcdReadReg(u8 Addr, u8 Len);

/******************************************************************************
  : LcdWriteReg
  : LcdдĴ
  : Addr - дַ
        pData - ͵ָ
        Len - ͵ݳ
  : 
******************************************************************************/
extern void LcdWriteVar(u16 Addr, u8 *pData, u8 Len);

/******************************************************************************
  : LcdReadVar
  : LcdȲܳ32,ͷļ֡ṹ
  : Addr - ַ
        Len - ݳ(Len,ǰWord)
  : 
******************************************************************************/
extern void LcdReadVar(u16 Addr, u8 Len);

/******************************************************************************
  : LcdShowPage
  : Lcdʾĳҳ
  : Page - ҳ
  : 
******************************************************************************/
extern void LcdShowPage(u16 Page);

/******************************************************************************
  : LcdGetPage
  : Lcdʾĳҳ
  : Page - ҳ
  : 
******************************************************************************/
void LcdGetPage(void);

/******************************************************************************
  : LcdShowNum
  : Lcdʾֵ
  : Addr - ʾλ
        Page - ҳ
  : 
******************************************************************************/
extern void LcdShowNum(u16 Addr, u16 Num);


/******************************************************************************
  : LcdShowStr
  : Lcdʾֵ
  : Addr - ʾλ
        pStr - ʾַ
        Len - ַ
  : 
******************************************************************************/
extern void LcdShowStr(u16 Addr, u8 *pStr, u8 Len);

/******************************************************************************
  : LcdSetRtc
  : Lcdʾֵ
  : pTime - ʱ
  : 
******************************************************************************/
extern void LcdSetRtc(StructTime *pTime);

/******************************************************************************
  : LcdGetPage
  : Lcdʾĳҳ
  : Page - ҳ
  : 
******************************************************************************/
void LcdGetRtc(void);

/******************************************************************************
  : LcdSetBackLight
  : Lcdʾֵ
  : Addr - ʾλ
        pStr - ʾַ
        Len - ַ
  : 
******************************************************************************/
extern void LcdSetBackLight(u8 Level);

/******************************************************************************
  : LcdBuzzer
  : Lcd
  : Ms - ʱ 10*Ms
  : 
******************************************************************************/
extern void LcdBuzzer(u8 Ms);

/******************************************************************************
  : LcdTouchCorrect
  : LcdУ
  : 
  : 
******************************************************************************/
extern void LcdTouchCorrect(void);

/******************************************************************************
  : LcdReset
  : Lcdλ
  : 
  : 
******************************************************************************/
extern void LcdReset(void);

/******************************************************************************
  : LcdShowData16
  : Lcdʾֵ
  : Addr - ʾλ
        Page - ҳ
  : 
******************************************************************************/
extern void LcdShowData16(u16 Addr, u16 Data16);

/******************************************************************************
  : LcdShowData32
  : Lcdʾֵ
  : Addr - ʾλ
        Page - ҳ
  : 
******************************************************************************/
extern void LcdShowData32(u16 Addr, u32 Data32);

#endif

